import os

from idm_test.dtk_test.integration.integration_test import IntegrationTest
from idm_test.dtk_test.integration import bootstrap, manifest


def update_sim_random_bic(simulation, alpha, beta):
    """
    The config parameter sweep function
    """
    simulation.task.config.parameters.Shedding_Distribution_Alpha = alpha
    simulation.task.config.parameters.Shedding_Distribution_Beta = beta
    return {"Shedding_Distribution_Alpha": alpha,
            "Shedding_Distribution_Beta": beta}


def set_param_fn(config):
    """
    Update the config parameters from default values.
    """
    print("Setting params.")
    config.parameters.Base_Infectivity_Constant = 0.6
    config.parameters['Config_Name'] = 'test_Enable_Nonuniform_Shedding'
    config.parameters.Incubation_Period_Constant = 0
    config.parameters.Infectious_Period_Constant = 100
    config.parameters.Simulation_Duration = 100
    config.parameters['Python_Script_Path'] = 'LOCAL'
    config.parameters['logLevel_Individual'] = 'DEBUG'
    return config


def build_camp():
    """
    Build a campaign input file for the DTK using emod_api.
    """
    import emod_api.campaign as camp
    import emod_api.interventions.outbreak as ob

    camp.set_schema(manifest.schema_file)
    ob_event = ob.seed_by_coverage(campaign_builder=camp, timestep=2, coverage=1)

    camp.add(ob_event)
    return camp


def build_demog():
    """
    Build a demographics input file for the DTK using emod_api.demographics or emodpy_**disease**.demographics
    """
    import emodpy_generic.demographics.GenericDemographics as Demographics  # OK to call into emod-api

    demog = Demographics.fromBasicNode(lat=0, lon=0, pop=100, name=1, forced_id=1)
    return demog


class TestNonuniformShedding(IntegrationTest):
    def setUp(self):
        # test_name is optional but recommended
        self.test_name = 'Test_Enable_Nonuniform_Shedding'
        # Copy Singularity definition files from idm_test package to your local folder(default value is './../sif').
        CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))
        sif_dir = os.path.join(CURRENT_DIR, "..", "sif")
        bootstrap.setup(local_dir=sif_dir)

    def test_nonuniform_shedding_sft(self):
        self.run_test(camp_fn_cb=build_camp, demog_fn_cb=build_demog, sweep_fn_cb=update_sim_random_bic,
                      sweep_values=([1, 2, 3], [1, 1.5, 4]), manifest=manifest, set_param_fn=set_param_fn,
                      force_build_exe=False)


if __name__ == '__main__':
    import unittest
    unittest.main()
