import os

from idm_test.dtk_test.integration.integration_test import IntegrationTest
from idm_test.dtk_test.integration import bootstrap, manifest
from tests.test_utils import generate_multiple_outbreak_waves

manifest.n_sims = 1


def update_sim_random_seed(simulation, value):
    simulation.task.config.parameters.Run_Number = value
    return {"Run_Number": value}


def set_param_fn(config):
    """
    Update the config parameters from default values.
    """
    print("Setting params.")
    config.parameters.Enable_Demographics_Reporting = 1
    config.parameters.Start_Time = 0  # default is 1
    config.parameters.Base_Infectivity_Constant = 0.7
    config.parameters.Base_Mortality = 0
    config.parameters['Config_Name'] = 'test_Single_Property_Within_Group_Target_Low_Group'
    config.parameters.Incubation_Period_Constant = 0
    config.parameters.Infectious_Period_Constant = 60
    config.parameters.Simulation_Duration = 60
    config.parameters.Enable_Property_Output = 1
    config.parameters.Symptomatic_Infectious_Offset = 0
    config.parameters['Python_Script_Path'] = 'LOCAL'
    config.parameters['logLevel_Individual'] = 'DEBUG'

    return config


def build_camp():
    """
    Build a campaign input file for the DTK using emod_api.
    Right now this function creates the file and returns the filename. If calling code just needs an asset that's fine.
    """
    import emod_api.campaign as camp
    camp.set_schema(manifest.schema_file)

    # Only target Risk:Low_Group for outbreak
    camp = generate_multiple_outbreak_waves(camp=camp, start_day=[0],
                                            property_group=["Risk:Low_Group"],
                                            coverage=0.05)
    return camp


def build_demog():
    """
    Build a demographics input file for the DTK using emod_api.
    Right now this function creates the file and returns the filename. If calling code just needs an asset that's fine.
    Also right now this function takes care of the config updates that are required as a result of specific demog settings.
    We do NOT want the emodpy-disease developers to have to know that. It needs to be done automatically in emod-api as much as possible.
    TBD: Pass the config (or a 'pointer' thereto) to the demog functions or to the demog class/module.
    """
    import emod_api.demographics.Demographics as Demographics

    demog = Demographics.from_template_node(lat=0, lon=0, pop=10000, name=1, forced_id=1)
    demog.SetBirthRate(0)
    demog.SetInitialAgeExponential(0.000118)
    demog.AddIndividualPropertyAndHINT(Property="Risk", Values=["High_Group", "Low_Group"],
                                       InitialDistribution=[0.5, 0.5],
                                       TransmissionMatrix=[[200, 0.0],
                                                           [0.0, 0.4]])

    return demog


class TestSinglePropertyWithinGroupTargetOneGroup(IntegrationTest):
    def setUp(self):
        # test_name is optional but recommended
        self.test_name = 'Test_Single_Property_Within_Group_Target_Low_Group'
        # Copy Singularity definition files from idm_test package to your local folder(default value is './../sif').
        CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))
        sif_dir = os.path.join(CURRENT_DIR, "..", "..", "sif")
        bootstrap.setup(sif_dir)

    def test_single_property_within_group_target_one_group_sft(self):
        self.run_test(camp_fn_cb=build_camp, demog_fn_cb=build_demog, sweep_fn_cb=update_sim_random_seed,
                      sweep_values=list(range(manifest.n_sims)), manifest=manifest, set_param_fn=set_param_fn,
                      force_build_exe=False)


if __name__ == '__main__':
    import unittest
    unittest.main()
