import os
from idm_test.dtk_test.integration.integration_test import IntegrationTest
from idm_test.dtk_test.integration import bootstrap, manifest


manifest.n_sims = 1


def set_param_fn(config):
    print("Setting params.")
    config.parameters.Enable_Birth = 0
    config.parameters.Enable_Natural_Mortality = 0
    config.parameters.Simulation_Duration = 100
    config.parameters['Python_Script_Path'] = 'LOCAL'
    config.parameters['Config_Name'] = 'test_Population_Division'
    config.parameters.Enable_Property_Output = 1
    config.parameters.Enable_Interventions = 0
    config.parameters.Incubation_Period_Constant = 0
    config.parameters.Infectious_Period_Constant = 1
    config.parameters.Base_Infectivity_Constant = 0.6

    return config


def build_camp():
    """
    Build a campaign input file for the DTK using emod_api.
    Right now this function creates the file and returns the filename. If calling code just needs an asset that's fine.
    """
    import emod_api.campaign as camp
    import emod_api.interventions.outbreak as ob

    camp.set_schema(manifest.schema_file)

    ob_event = ob.seed_by_coverage(campaign_builder=camp, timestep=30, coverage=0.001)

    camp.add(ob_event)
    return camp


def build_demog():
    """
    Build a demographics input file for the DTK using emod_api.
    Right now this function creates the file and returns the filename. If calling code just needs an asset that's fine.
    Also right now this function takes care of the config updates that are required as a result of specific demog settings.
    We do NOT want the emodpy-disease developers to have to know that. It needs to be done automatically in emod-api as much as possible.
    TBD: Pass the config (or a 'pointer' thereto) to the demog functions or to the demog class/module.
    """
    import emodpy_generic.demographics.GenericDemographics as Demographics  # OK to call into emod-api

    demog = Demographics.fromBasicNode(lat=0, lon=0, pop=100, name=1, forced_id=1)
    demog.AddIndividualPropertyAndHINT(Property="Accessibility",
                                       Values=["Easy", "Hard"],
                                       InitialDistribution=[0.7, 0.3])
    demog.AddIndividualPropertyAndHINT(Property="Risk",
                                       Values=["Low", "High"],
                                       InitialDistribution=[0.8, 0.2])
    return demog


class TestPopulationDivision(IntegrationTest):
    def setUp(self):
        self.test_name = 'Test_Population_Division'
        CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))
        sif_dir = os.path.join(CURRENT_DIR, "..", "..", "sif")
        bootstrap.setup(sif_dir)

    def test_population_division_sft(self):
        self.run_test(camp_fn_cb=build_camp, demog_fn_cb=build_demog, sweep_fn_cb=None, manifest=manifest,
                      set_param_fn=set_param_fn, force_build_exe=False)


