import os

import emod_api.campaign as camp
from emod_api.interventions.common import ScheduledCampaignEvent
from idm_test.dtk_test.integration import bootstrap, manifest
from idm_test.dtk_test.integration.integration_test import IntegrationTest

manifest.n_sims = 1


def update_sim_random_seed(simulation, value):
    simulation.task.config.parameters.Run_Number = value
    return {"Run_Number": value}


def set_param_fn(config):
    print("Setting params.")
    config.parameters.Enable_Birth = 0
    config.parameters.Start_Time = 0  # default is 1
    config.parameters.Enable_Natural_Mortality = 1
    config.parameters.Enable_Vital_Dynamics = 0
    config.parameters.Simulation_Duration = 25
    config.parameters['Python_Script_Path'] = 'LOCAL'
    config.parameters['Config_Name'] = 'Test_Property_Targeting'
    config.parameters.Enable_Property_Output = 1
    config.parameters.Enable_Interventions = 0
    config.parameters.Incubation_Period_Constant = 0
    config.parameters.Infectious_Period_Distribution = "EXPONENTIAL_DISTRIBUTION"
    config.parameters.Infectious_Period_Exponential = 7
    config.parameters.Base_Infectivity_Constant = 0.7

    return config


def build_camp():
    """
    Build a campaign input file for the DTK using emod_api.
    """
    import emod_api.interventions.simple_vaccine as sv
    camp.set_schema(manifest.schema_file)
    sv.schema_path = manifest.schema_file

    event_vc_easy = ScheduledCampaignEvent(camp=camp, Start_Day=1, Property_Restrictions=["Accessibility:Easy"],
                                           Demographic_Coverage=1.0,
                                           Intervention_List=[
                                               sv.new_intervention(timestep=1, v_type="Acquire", efficacy=0,
                                                                   intervention_only=True, waning_duration=3650,
                                                                   cost_to_consumer=1)])
    event_vc_hard = ScheduledCampaignEvent(camp=camp, Start_Day=2, Property_Restrictions=["Accessibility:Hard"],
                                           Demographic_Coverage=1.0,
                                           Intervention_List=[
                                               sv.new_intervention(timestep=2, v_type="Acquire", efficacy=0,
                                                                   intervention_only=True, waning_duration=3650,
                                                                   cost_to_consumer=1)])

    event_vc_low = ScheduledCampaignEvent(camp=camp, Start_Day=11, Property_Restrictions=["Risk:Low"],
                                          Demographic_Coverage=1.0,
                                          Intervention_List=[
                                              sv.new_intervention(timestep=11, v_type="Acquire", efficacy=0,
                                                                  intervention_only=True, waning_duration=3650,
                                                                  cost_to_consumer=1)])

    event_vc_high = ScheduledCampaignEvent(camp=camp, Start_Day=12, Property_Restrictions=["Risk:High"],
                                           Demographic_Coverage=1.0,
                                           Intervention_List=[
                                               sv.new_intervention(timestep=12, v_type="Acquire", efficacy=0,
                                                                   intervention_only=True, waning_duration=3650,
                                                                   cost_to_consumer=1)])
    event_vc_easy_low = ScheduledCampaignEvent(camp=camp, Start_Day=21,
                                               Property_Restrictions=["Accessibility:Easy", "Risk:Low"],
                                               Demographic_Coverage=1.0,
                                               Intervention_List=[
                                                   sv.new_intervention(timestep=21, v_type="Acquire", efficacy=0,
                                                                       intervention_only=True, waning_duration=3650,
                                                                       cost_to_consumer=1)])
    event_vc_easy_high = ScheduledCampaignEvent(camp=camp, Start_Day=22,
                                                Property_Restrictions=["Accessibility:Easy", "Risk:High"],
                                                Demographic_Coverage=1.0,
                                                Intervention_List=[
                                                    sv.new_intervention(timestep=22, v_type="Acquire", efficacy=0,
                                                                        intervention_only=True, waning_duration=3650,
                                                                        cost_to_consumer=1)])

    event_vc_hard_high = ScheduledCampaignEvent(camp=camp, Start_Day=24,
                                                Property_Restrictions=["Accessibility:Hard", "Risk:High"],
                                                Demographic_Coverage=1.0,
                                                Intervention_List=[
                                                    sv.new_intervention(timestep=24, v_type="Acquire", efficacy=0,
                                                                        intervention_only=True, waning_duration=3650,
                                                                        cost_to_consumer=1)])

    camp.add(event_vc_easy)
    camp.add(event_vc_hard)
    camp.add(event_vc_low)
    camp.add(event_vc_high)
    camp.add(event_vc_easy_low)
    camp.add(event_vc_easy_high)
    camp.add(event_vc_hard_high)
    return camp


def build_demog():
    import emod_api.demographics.Demographics as Demographics

    demog = Demographics.from_template_node(lat=0, lon=0, pop=10000, name=1, forced_id=1)
    demog.SetBirthRate(0)
    demog.SetInitialAgeExponential(0.000118)
    demog.AddIndividualPropertyAndHINT(Property="Accessibility",
                                       Values=["Easy", "Hard"],
                                       InitialDistribution=[0.7, 0.3])
    demog.AddIndividualPropertyAndHINT(Property="Risk",
                                       Values=["Low", "High"],
                                       InitialDistribution=[0.8, 0.2])
    demog.SetMortalityRate(0.020002, node_ids=[1])
    demog.SetInitPrevFromUniformDraw(min_init_prev=0, max_init_prev=0)
    return demog


class TestPropertyTarget(IntegrationTest):
    def setUp(self):
        # test_name is optional but recommended
        self.test_name = 'Test_Property_Targeting'
        # Copy Singularity definition files from idm_test package to your local folder(default value is './../sif').
        CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))
        sif_dir = os.path.join(CURRENT_DIR, "..", "..", "sif")
        bootstrap.setup(sif_dir)

    def test_property_targeting_sft(self):
        self.run_test(camp_fn_cb=build_camp, demog_fn_cb=build_demog, sweep_fn_cb=update_sim_random_seed,
                      sweep_values=list(range(manifest.n_sims)), manifest=manifest, set_param_fn=set_param_fn,
                      force_build_exe=False)


if __name__ == '__main__':
    import unittest

    unittest.main()