import os

from idm_test.dtk_test.integration.integration_test import IntegrationTest
from idm_test.dtk_test.integration import bootstrap, manifest

manifest.n_sims = 1


def update_sim_random_bic(simulation, value):
    simulation.task.config.parameters.Base_Infectivity_Constant = value
    return {"Base_Infectivity_Constant": value}


def set_param_fn(config):
    print("Setting params.")
    config.parameters.Start_Time = 0  # default is 1
    config.parameters.Base_Individual_Sample_Rate = 1
    config.parameters.Base_Infectivity_Constant = 0.75
    config.parameters.Base_Mortality = 0
    config.parameters.Enable_Immune_Decay = 0
    config.parameters.Enable_Natural_Mortality = 0
    config.parameters.Enable_Vital_Dynamics = 0
    config.parameters['Config_Name'] = 'Test_InfectionRate_Basic_Calculation'
    config.parameters.Incubation_Period_Constant = 0
    config.parameters.Infectious_Period_Constant = 50
    config.parameters.Simulation_Duration = 50
    config.parameters.x_Base_Population = 0.1
    config.parameters['Python_Script_Path'] = 'LOCAL'
    config.parameters['logLevel_Individual'] = 'DEBUG'

    return config


def build_camp():
    """
    Build a campaign input file for the DTK using emod_api.
    Right now this function creates the file and returns the filename. If calling code just needs an asset that's fine.
    """
    import emod_api.campaign as camp
    import emod_api.interventions.outbreak as ob

    camp.set_schema(manifest.schema_file)

    ob_event = ob.seed_by_coverage(campaign_builder=camp, timestep=0, coverage=0.1)

    camp.add(ob_event)
    return camp


def build_demog():
    """
    Build a demographics input file for the DTK using emod_api.
    """
    import emod_api.demographics.Demographics as Demographics

    demog = Demographics.from_template_node(lat=0, lon=0, pop=1000, name=1, forced_id=1)
    demog.SetBirthRate(0)
    demog.SetInitialAgeExponential(0.000118)
    #demog.SetInitPrevFromUniformDraw(min_init_prev=0, max_init_prev=0)

    return demog


class TestBasicInfection(IntegrationTest):

    def setUp(self):
        # test_name is optional but recommended
        self.test_name = 'Test_InfectionRate_Basic_Calculation'
        # Copy Singularity definition files from idm_test package to your local folder(default value is './../sif').
        CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))
        sif_dir = os.path.join(CURRENT_DIR, "..", "sif")
        bootstrap.setup(sif_dir)

    def test_basic_infection_rate_calculation_sft(self):
        self.run_test(camp_fn_cb=build_camp, demog_fn_cb=build_demog, sweep_fn_cb=update_sim_random_bic,
                      sweep_values=([0.75, 1, 0, 1000]), manifest=manifest, set_param_fn=set_param_fn,
                      force_build_exe=False)


if __name__ == '__main__':
    import unittest
    unittest.main()